package com.unis.detectionnum.server.controller;

import com.unis.detectionnum.server.domain.LogDomain;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import com.github.pagehelper.PageInfo;
import com.unis.common.utils.ResponseWrapper;
import com.unis.detectionnum.common.vo.AlarmParamVO;
import com.unis.detectionnum.common.vo.DCfxxsjyVO;
import com.unis.detectionnum.common.vo.LogVO;
import com.unis.detectionnum.server.service.AlarmParamService;
import com.unis.detectionnum.server.service.LogService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

/** 
 * @ClassName: LogController 
 * @Description: 平台管理-操作日志管理 
 * @author LiuFeng 
 * @date 2019年10月16日 下午4:55:23  
 */
@RestController
@Slf4j
@Api(value = "平台管理-操作日志管理", tags = "平台管理-操作日志管理")
public class LogController {

	@Autowired
	private LogService logService;

	@Autowired
	private LogDomain logDomain;
	
	/**
	 * @Description 获取操作日志信息列表
	 * @param 操作人名称
	 * @return 企业信息列表
	 */
	@ApiOperation(value = "分页获取操作日志信息列表", notes = "参数：操作人名称")
	@PostMapping(value = "/logs/pageList/{pageNum}/{pageSize}")
	public ResponseWrapper logsPageList(@PathVariable("pageNum")Integer pageNum,@PathVariable("pageSize") Integer pageSize,
			@ApiParam(name="userName",value="账户",required=false)
			@RequestParam(value = "userName", required = false)String userName) {
		try {
			PageInfo<LogVO> pageInfos = logService.getLogsPageList(userName,pageNum,pageSize) ;
            return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), pageInfos);
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }
		return ResponseWrapper.error(HttpStatus.INTERNAL_SERVER_ERROR.value(), HttpStatus.INTERNAL_SERVER_ERROR.getReasonPhrase(),"服务错误，请联系管理员");
	}

	/**
	 * 新增
	 *
	 * @param logVO
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/logs/insertLog")
	public ResponseWrapper insertLog(@RequestBody LogVO logVO){
		int save = logDomain.save(logVO);
		return ResponseWrapper.success(HttpStatus.OK.value(), HttpStatus.OK.getReasonPhrase(), save);
	}
	
}
