package com.unis.detectionnum.server.schedule;

import com.unis.detectionnum.common.vo.AlarmParamVO;
import com.unis.detectionnum.common.vo.DeviceAlarmVO;
import com.unis.detectionnum.server.domain.AlarmParamDomain;
import com.unis.detectionnum.server.entity.DeviceInfoDO;
import com.unis.detectionnum.server.service.DeviceAlarmService;
import com.unis.detectionnum.server.service.DeviceParamService;
import com.unis.detectionnum.server.service.ScanPointsService;
import com.unis.detectionnum.server.utils.CommonDataMeasure;
import com.unis.detectionnum.server.utils.UUIDUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 文件描述:定时查看设备是否在线，并发送报警消息
 *
 * @author tongxi.xia
 * @date 2020年06月04日 10:37
 */
@Service
public class SendAlarmMessage {
    private static final Logger log = LoggerFactory.getLogger(SendAlarmMessage.class);

    @Autowired
    private DeviceAlarmService deviceAlarmService;

    @Autowired
    private DeviceParamService deviceParamService;

    @Autowired
    private ScanPointsService scanPointsService;

    @Autowired
    AlarmParamDomain alarmParamDomain;

//    @Scheduled(cron = "0 */30 * * * ?")
    public void execute() {
        log.info("每30分钟执行一次，定时查看设备是否在线，并发送报警消息");
        List<DeviceInfoDO> deviceInfoDOS = deviceParamService.selectListAll();
        if (deviceInfoDOS.size() > 0) {
            for (int i = 0; i < deviceInfoDOS.size(); i++) {
                DeviceInfoDO deviceInfoDO = deviceInfoDOS.get(i);
                String sbCode = deviceInfoDO.getSbCode();
                String sbIpdz = deviceInfoDO.getSbZwym();
                boolean ping = ping(sbIpdz, 2, 5000);
                if (!ping) {
                    //连接失败，保存告警信息
                    DeviceAlarmVO deviceAlarmVO = new DeviceAlarmVO();
                    deviceAlarmVO.setCreateDate(new Date());
                    deviceAlarmVO.setType(2);
                    deviceAlarmVO.setSbCode(sbCode);
                    deviceAlarmVO.setRemark("探头连接失败");
                    deviceAlarmVO.setId(UUIDUtils.getUUID());
                    deviceAlarmService.saveDeviceAlarm(deviceAlarmVO);
                    //发送短信报警
                    AlarmParamVO alarmParamVO = alarmParamDomain.getAlarmParamBySbCode(sbCode);
                    scanPointsService.sendAlarm(alarmParamVO, CommonDataMeasure.MSG2);
                    log.info("设备ping连接异常,设备code: " + sbCode);
                }
            }
        }
    }

    /**
     * 也是使用java调用控制台的ping命令，这个比较可靠，还通用，使用起来方便：传入个ip，设置ping的次数和超时，就可以根据返回值来判断是否ping通。
     *
     * @param ipAddress
     * @param pingTimes
     * @param timeOut
     * @return
     */
    private static boolean ping(String ipAddress, int pingTimes, int timeOut) {
        BufferedReader in = null;
        Runtime r = Runtime.getRuntime();  // 将要执行的ping命令,此命令是windows格式的命令
        String pingCommand = "ping " + ipAddress + " -n " + pingTimes + " -w " + timeOut;
        try {   // 执行命令并获取输出
            log.info("执行命令并获取输出:" + pingCommand);
            Process p = r.exec(pingCommand);
            if (p == null) {
                return false;
            }
            in = new BufferedReader(new InputStreamReader(p.getInputStream()));   // 逐行检查输出,计算类似出现=23ms TTL=62字样的次数
            int connectedCount = 0;
            String line = null;
            while ((line = in.readLine()) != null) {
                connectedCount += getCheckResult(line);
            }   // 如果出现类似=23ms TTL=62这样的字样,出现的次数=测试次数则返回真
            return connectedCount == pingTimes;
        } catch (Exception ex) {
            ex.printStackTrace();   // 出现异常则返回假
            return false;
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //若line含有=18ms TTL=16字样,说明已经ping通,返回1,否則返回0.
    private static int getCheckResult(String line) {  // System.out.println("控制台输出的结果为:"+line);
        Pattern pattern = Pattern.compile("(\\d+ms)(\\s+)(TTL=\\d+)", Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(line);
        while (matcher.find()) {
            return 1;
        }
        return 0;
    }
}
